/****************************************************************
Project : PPE SERRE
Version : 1
// --------------------------------------------------------------
Fichier source : SERREFIN.c
Date de  cration : 15/12/2010
Mise  jour le : 09/03/2012
// --------------------------------------------------------------
Author  : Philippe Mariano
Company : Lyce Pierre Emile Martin 18000 BOURGES
// --------------------------------------------------------------
Comments: PPE SERRE 
// --------------------------------------------------------------
Cross compilateur: CVAVR V2.04.08a Standard
// --------------------------------------------------------------
// Cible hardware : Carte SISI (I2C soft)
// --------------------------------------------------------------
Chip type           : ATmega32
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 512
****************************************************************/
//===============================================================================================================
// Bibliothques
//---------------------------------------------------------------------------------------------------------------
// CVAVR
//---------------------------------------------------------------------------------------------------------------
#include <mega32.h>
//---------------------------------------------------------------------------------------------------------------
#include <stdio.h>      // contient la fonction sprintf
#include <delay.h>      // contient la fonction delay_ms
#include <stdlib.h> 	// contient la fonction abs()
//---------------------------------------------------------------------------------------------------------------
// Bibliothques dveloppes pour la carte SISI
//---------------------------------------------------------------------------------------------------------------
#include "SISI2.h"          // Contient les fonctions: Lire_BP_I2C_Soft, Affiche_LCD, Arret_Moteurs,
                            // Active_Moteurs, Affiche_Fleche, RegleHTR
#include "PCF8574.h"        // Contient les fonctions: Ecrire_Port_I2C_Soft, Lire_Port_I2C_Soft
//===============================================================================================================
// I2C Bus functions
#asm
   .equ __i2c_port=0x18 ;PORTB
   .equ __sda_bit=1
   .equ __scl_bit=0
#endasm
#include <i2c.h>
//---------------------------------------------------------------------------------------------------------------
// DS1621 Thermometer/Thermostat functions
#include <ds1621.h>

// PCF8583 Real Time Clock functions
#include <pcf8583.h>
//---------------------------------------------------------------------------------------------------------------
// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15 ;PORTC
#endasm
#include <lcd.h>
 //--------------------------------------------------------------------------------------------------------------
// #include <alcd.h>   // PB avec biblio SSI2 !
//---------------------------------------------------------------------------------------------------------------
#define ADC_VREF_TYPE 0x20
//===============================================================================================================

//===============================================================================================================
// Dclarations
//---------------------------------------------------------------------------------------------------------------
// Etats du graphe des menus
//---------------------------------------------------------------------------------------------------------------
#define ReglerHTR                   0
#define Afficher                    1
#define Tester                      2
#define Simuler                     3
#define Reguler                     4
#define AfficherHorloge             5
#define AfficherMesures             6
#define TesterChauffage             7
#define TesterVentilation           8
#define TesterTrappes               9
#define CommanderChauffageManu      10
#define CommanderVentilationManu    11
#define CommanderTrappesManu        12
#define RegulationTOR               14

//---------------------------------------------------------------------------------------------------------------
// Symboles LCD personaliss
//---------------------------------------------------------------------------------------------------------------
#define FlecheH                     0
#define FlecheB                     1
#define Soleil                      2
#define Lune                        3
//---------------------------------------------------------------------------------------------------------------
// Adresse HTR
//---------------------------------------------------------------------------------------------------------------
#define AdresseHTR                  0
//---------------------------------------------------------------------------------------------------------------
// Entres du CAN
//---------------------------------------------------------------------------------------------------------------
#define EntreeHumidite              6
#define EntreeTemperatureSerre      5
#define EntreeLuminositee           0
//---------------------------------------------------------------------------------------------------------------
// Sorties PWM
//---------------------------------------------------------------------------------------------------------------
#define PWMChauffage                OCR1B
#define PWMTrappes                  PORTD.7
#define TORVentilation              PORTD.5
//---------------------------------------------------------------------------------------------------------------
// Visualisations sur les leds de la carte SISI
#define LedsEteintes                0b11111111
#define VisuCdeChauffage            0b01111111
#define VisuCdeVentilation          0b10111111
#define VisuCdeTrappes              0b11011111
#define VisuLectureTemps            0b11111110
#define VisuLectureTempInt          0b11111101
#define VisuLectureTempExt          0b11111011
#define VisuLectureHumid            0b11110111
#define VisuLectureLum              0b11101111
//---------------------------------------------------------------------------------------------------------------
// Rglage de l'HTR par dfaut
//---------------------------------------------------------------------------------------------------------------
#define   ChoixHeure                0
#define   ChoixDate                 1
#define   DefautHeure               12
#define   DefautMinute              30
#define   DefautSeconde             0
#define   DefautAnnee               2013
#define   DefautMois                6
#define   DefautJour                15
//===============================================================================================================

//===============================================================================================================
// Read the 8 most significant bits
// of the AD conversion result
unsigned char read_adc(unsigned char adc_input)
{
ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
// Delay needed for the stabilization of the ADC input voltage
delay_us(10);
// Start the AD conversion
ADCSRA|=0x40;
// Wait for the AD conversion to complete
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCH;
}
// ==============================================================================================================

// ==============================================================================================================
//---------------------------------------------------------------------------------------------------------------
// Declare your global variables here
//---------------------------------------------------------------------------------------------------------------
// Type 		            Nom 				       		 Description
//---------------------------------------------------------------------------------------------------------------
bit AfficheMenuReglageHTR = 0;                         //  Gestion de l'affichage
bit PremierReglageHeureHTR = 1;
bit PremierReglageDateHTR = 1;
//---------------------------------------------------------------------------------------------------------------
// Constantes en flash
//---------------------------------------------------------------------------------------------------------------
flash char TexteMenuAcquiter[]      = "         [ENTR] ";
flash char TexteMenuAcquitQuitter[] = " [ENTR] [ECHAP] ";
flash char TexteMenuReglerHTR[]     = "[1] Reglage HTR ";
flash char TexteMenusAfficher[]	    = "[2] Affichages  ";
flash char TexteMenusTester[]       = "[3] Tests       ";
flash char TexteMenuRegulerTOR[]    = "[4] Regul. TOR  ";
flash char TexteMenuHeureOuDate[]   = " SET>H ETR>Date ";
flash char TexteMenuPlusMoinsEsc[]  = "[+] [-] [ECHAP] ";
flash char TexteSousMenuChauffage[]     = "[31] Test Chauf ";
flash char TexteSousMenuVentilation[]   = "[32] Test Vent. ";
flash char TexteSousMenuTrappes[]       = "[33] Test Trap. ";
//---------------------------------------------------------------------------------------------------------------
// Tables des caractres personnaliss:  placer dans la mmoire CGRAM du LCD (8 caractres maxi)
//---------------------------------------------------------------------------------------------------------------
flash unsigned char IconeFlecheHaut[] = {0x04, 0x0E, 0x15, 0x04, 0x04, 0x04, 0x04, 0x00};   // @1 CGRAM
flash unsigned char IconeFlecheBas[]  = {0x04, 0x04, 0x04, 0x04, 0x15, 0x0E, 0x04, 0x00};   // @2 CGRAM
flash unsigned char IconeJour[]       = {0x04, 0x0A, 0x11, 0x04, 0x11, 0x0A, 0x04, 0x00};   // @3 CGRAM
flash unsigned char IconeNuit[]       = {0x00, 0x06, 0x07, 0x07, 0x07, 0x06, 0x00, 0x00};   // @4 CGRAM
// ==============================================================================================================

// ==============================================================================================================
// Prototypes des fonctions en cours de test
//---------------------------------------------------------------------------------------------------------------
void AfficheCaracterePersoSurLCD(unsigned char numero, unsigned char posColonne, unsigned char posLigne);
unsigned char ReglagePWMServo(unsigned int Reglaget1Servo);
// ==============================================================================================================

// ==============================================================================================================
// ==============================================================================================================
void main(void)
{
// ==============================================================================================================
// Declare your local variables here
//---------------------------------------------------------------------------------------------------------------
// Type 		            Nom 				       		 Description
//---------------------------------------------------------------------------------------------------------------
// Liaison srie SISI -> PC
char TrameRS232Mesures[70];                                             // Tableau des mesures  afficher sur le PC
unsigned int NumeroMesure = 0;
//---------------------------------------------------------------------------------------------------------------
// Afficheur LCD  processeur Hitachi
char display_buffer_ligne0[17]; 				                        // Buffer ligne 0 de l'afficheur
char display_buffer_ligne1[17]; 	       	       		                // Buffer ligne 1 de l'afficheur
//---------------------------------------------------------------------------------------------------------------
// HTR
unsigned char Jour, Mois = 0;                                           // Date de la HTR
unsigned int Annee = 0;
unsigned char Heures = 0, Minutes = 0, Secondes_n = 0, Dizieme = 0;     // Heure de la HTR
unsigned char Secondes_n_1 = 0, Secondes_n_1H = 0; 	                    // Assure la stabilit de l'affichage
//---------------------------------------------------------------------------------------------------------------
// BP de la carte SSI
unsigned char BP = 0; // Code_BP = 0;
//---------------------------------------------------------------------------------------------------------------
// Graphe des transitions des menus
unsigned char EtatMenu = ReglerHTR, EtatMenu_1 = AfficherHorloge;
//---------------------------------------------------------------------------------------------------------------
// Grandeurs physiques
//---------------------------------------------------------------------------------------------------------------
// Temps
unsigned char DeltaTemps;
//---------------------------------------------------------------------------------------------------------------
// Humidit
unsigned char UHumidite = 0;     			// Image de la tension fournie par le capteurl'humidit [0, 255]
float RH_n = 0, RH_n_1 = 0;				    // Valeur de l'humidit calcule [0, 100%]
//---------------------------------------------------------------------------------------------------------------
// Temprature
unsigned char UTemperatureInterieure = 0;   // Image de la tension dlivre par la sonde PT100
unsigned int TemperatureExterieure_n_1 = 0, TemperatureExterieure_n = 0;   // Valeur de la temprature extrieure
float SeuilTempIntSimuleMin = 21.5, SeuilTempIntSimuleMax = 22.5;
float TemperatureInterieure_n = 0, TemperatureInterieure_n_1 = 0;
//---------------------------------------------------------------------------------------------------------------
// Luminosit
unsigned char Luminosite = 0;               // Image de la Luminosit [0,255]
unsigned char JourOuNuit_n_1 = 0, JourOuNuit_n = 1;
//---------------------------------------------------------------------------------------------------------------
// Chanes de caractres pour l'affichage des grandeurs physiques
unsigned char ChaineHumidite[]="00.0";
unsigned char ChaineJourNuit[]="Jour";
unsigned char ChaineTemperatureInterieure[]="00.0";
unsigned char ChaineSeuilTempIntSimuleMin[]="00.0";
unsigned char ChaineSeuilTempIntSimuleMax[]="00.0";
unsigned char ChaineCdeChauffage[]="000";
//---------------------------------------------------------------------------------------------------------------
// Compteurs associs  la mesure des grandeurs physiques
unsigned int CompteurMesureTemperatureExterieure = 0;
unsigned int CompteurMesureTemperatureInterieure = 0;
unsigned int CompteurMesureHumidite = 0;
unsigned int CompteurMesureLuminosite = 0;
unsigned int CompteurTxRS232 = 0;
//---------------------------------------------------------------------------------------------------------------
// Timer
unsigned char rcycl=0, rcycl_1=0;               // Consigne de rapport cyclique
// Table des coefficients pour le rglage du rapport cyclique du signal PWMChauffage (F = 3,8Hz)
unsigned int alphaChauffage[11] = {0,100,204,304,408,508,612,712,816,916,1023};
//---------------------------------------------------------------------------------------------------------------
// Commande TOR
unsigned char TORVentilation_1 = 1;
unsigned int t1Servo = 0, t1Servo_1 = 1;
unsigned char CommandeChauffage = 0;
//---------------------------------------------------------------------------------------------------------------
// Divers
unsigned char i=0;                             // Compteur de boucle
//---------------------------------------------------------------------------------------------------------------
// Constantes
//---------------------------------------------------------------------------------------------------------------
// ==============================================================================================================
// Initialisation des priphriques
//---------------------------------------------------------------------------------------------------------------
// Port D initialization
// Func7=Out Func6=In Func5=Out Func4=Out Func3=In Func2=In Func1=In Func0=In
// State7=0 State6=T State5=0 State4=0 State3=T State2=T State1=T State0=T
PORTD=0x00;
DDRD=0x10; //0xB0
//---------------------------------------------------------------------------------------------------------------
// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 7,813 kHz
// Mode: Ph. correct PWM top=03FFh
// OC1A output: Discon.
// OC1B output: Non-Inv.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x23;
TCCR1B=0x05;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;

ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;
//---------------------------------------------------------------------------------------------------------------
// ADC initialization
// ADC Clock frequency: 62,500 kHz
// ADC Voltage Reference: AREF pin
// ADC High Speed Mode: Off
// ADC Auto Trigger Source: ADC Stopped
// Only the 8 most significant bits of
// the AD conversion result are used
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x87;
SFIOR&=0xEF;
//---------------------------------------------------------------------------------------------------------------
// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: Off
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 9600
UCSRA=0x00;
UCSRB=0x08;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x33;
//---------------------------------------------------------------------------------------------------------------
// I2C Bus initialization
i2c_init();
//---------------------------------------------------------------------------------------------------------------
// DS1621 Thermometer/Thermostat initialization
// tlow: 50C
// thigh: 55C
// Tout polarity: 0
ds1621_init(0,50,55,0);
//---------------------------------------------------------------------------------------------------------------
// PCF8583 Real Time Clock initialization
rtc_init(0,0);
//---------------------------------------------------------------------------------------------------------------
// Alphanumeric LCD initialization
// Connections specified in the
// Project|Configure|C Compiler|Libraries|Alphanumeric LCD menu:
// RS - PORTC Bit 0
// RD - PORTC Bit 1
// EN - PORTC Bit 2
// D4 - PORTC Bit 4
// D5 - PORTC Bit 5
// D6 - PORTC Bit 6
// D7 - PORTC Bit 7
// Characters/line: 16
lcd_init(16);
//------------------------------------------------------------------------------------------------------------
// Chargement de la mmoire CGRAM de l'afficheur avec les caractres personaliss
//------------------------------------------------------------------------------------------------------------
for (i=0;i<8;i++)
{
 lcd_write_byte(Add_Base_CGRAM + i, IconeFlecheHaut[i]);            // Caractre 0 flche haut
 lcd_write_byte(Add_Base_CGRAM + 8 + i, IconeFlecheBas[i]);         // Caractre 1 flche bas
 lcd_write_byte(Add_Base_CGRAM + 16 + i, IconeJour[i]);             // Caractre 2 Jour
 lcd_write_byte(Add_Base_CGRAM + 24 + i, IconeNuit[i]);             // Caractre 3 Nuit
}
//------------------------------------------------------------------------------------------------------------
// ==============================================================================================================
// Message d'accueil
//---------------------------------------------------------------------------------------------------------------
sprintf(display_buffer_ligne0," PPE SERRE SISI ");
sprintf(display_buffer_ligne1," Maj:01/12/2013 ");
Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
// Initialisation des seuils de temprature
ftoa(SeuilTempIntSimuleMin,1,ChaineSeuilTempIntSimuleMin);
ftoa(SeuilTempIntSimuleMax,1,ChaineSeuilTempIntSimuleMax);
delay_ms(5000);

// ==============================================================================================================
// ==============================================================================================================
// Boucle principale
// ==============================================================================================================
// ==============================================================================================================
//---------------------------------------------------------------------------------------------------------------
while (1)
 {
// ==============================================================================================================
// Dbut acqurir
//---------------------------------------------------------------------------------------------------------------
// Lecture du clavier, de l'heure, de la temprature intrieure et extrieure  la serre, de l'humidit intrieure
//  la serre et de la luminosit
//---------------------------------------------------------------------------------------------------------------
    BP = Lire_BP_I2C_Soft();                            // Lecture BP pour volution du graphe des menus
                                                        // Avec gestion anti-rebonds
    //Code_BP =  Lire_Port_I2C_Soft(Add_BP_I2C);          // Lecture BP pour affichage sur les LED
                                                        // Code BP = Code LED
 //---------------------------------------------------------------------------------------------------------------
 // Lecture HTR
    rtc_get_time(AdresseHTR,&Heures,&Minutes,&Secondes_n,&Dizieme);

 // Rfrence de temps pour la mesure des grandeurs physiques
    if (Secondes_n_1 < Secondes_n)
                DeltaTemps = Secondes_n - Secondes_n_1;
    else if (Secondes_n_1 > Secondes_n)
                DeltaTemps = (59 - Secondes_n) + Secondes_n_1;
    else
                DeltaTemps = 0;

    if (DeltaTemps != 0)
     {
        // Mise  jour des compteurs associs aux mesures
        CompteurMesureTemperatureExterieure = CompteurMesureTemperatureExterieure + DeltaTemps;
        CompteurMesureTemperatureInterieure = CompteurMesureTemperatureInterieure + DeltaTemps;
        CompteurMesureHumidite = CompteurMesureHumidite + DeltaTemps;
        CompteurMesureLuminosite = CompteurMesureLuminosite + DeltaTemps;
        CompteurTxRS232 = CompteurTxRS232 + DeltaTemps;
        Secondes_n_1 = Secondes_n;   
        // Visualisation de la lecture de l'HTR sur la Led D0
        Ecrire_Port_I2C_Soft(Add_LED_I2C,VisuLectureTemps); // Visualisation de la prise de mesure
        delay_ms(50);
        Ecrire_Port_I2C_Soft(Add_LED_I2C,LedsEteintes);
//---------------------------------------------------------------------------------------------------------------           
// Acquisition de la temprature extrieure  la serre
        if (CompteurMesureTemperatureExterieure >= 5)   // A rgler pour mesure de la temprature toute les 5 mn
            {
                TemperatureExterieure_n = ds1621_temperature_10(0) - 20; // Correction =  -2C
                CompteurMesureTemperatureExterieure = 0;
                // Visualisation de la prise d'une mesure de la temprature extrieure sur la Led D1
                Ecrire_Port_I2C_Soft(Add_LED_I2C,VisuLectureTempExt); 
                delay_ms(50);
                Ecrire_Port_I2C_Soft(Add_LED_I2C,LedsEteintes);
            }  
//---------------------------------------------------------------------------------------------------------------        
// Acquisition de la temprature intrieure  la serre
        if (CompteurMesureTemperatureInterieure >= 5) // A rgler
            {
                UTemperatureInterieure =  read_adc(EntreeTemperatureSerre);
                TemperatureInterieure_n = (float)(UTemperatureInterieure)*0.109;//  Calcul de la temprature intrieure
                CompteurMesureTemperatureInterieure = 0;
                // Visualisation de la prise d'une mesure de la temprature intrieure sur la led D2
                Ecrire_Port_I2C_Soft(Add_LED_I2C,VisuLectureTempInt); 
                delay_ms(50);
                Ecrire_Port_I2C_Soft(Add_LED_I2C,LedsEteintes);
            }
//---------------------------------------------------------------------------------------------------------------        
// Acquisition de l'humidit intrieure  la serre
        if (CompteurMesureHumidite >= 10)   // Rgler la mesure d'humidit toutes les 2 mn
            {
                UHumidite = read_adc(EntreeHumidite);
                RH_n =  ((float)UHumidite - 42.64)/ 1.536;   // ((UHumidite - 41(35.84))/ 1.7064); Calcul de l'humidit
                CompteurMesureHumidite = 0;
                // Visualisation de la prise d'une mesure de l'humidit intrieure sur la led D3
                Ecrire_Port_I2C_Soft(Add_LED_I2C,VisuLectureHumid);
                delay_ms(50);
                Ecrire_Port_I2C_Soft(Add_LED_I2C,LedsEteintes);
            }
//---------------------------------------------------------------------------------------------------------------       
        // Acquisition de la luminosit
        if (CompteurMesureLuminosite >= 3)   // Rgler la mesure de la luminosit toutes les 10mn
            {
                Luminosite = read_adc(EntreeLuminositee);
                if (Luminosite >= 230)          // VPA1 > 4.5V
                    {
                        JourOuNuit_n = 0;
                        sprintf(ChaineJourNuit,"Jour");
                    }
                else if (Luminosite <= 165)     // VPA1 < 3.2V
                    {
                        JourOuNuit_n = 1;
                        sprintf(ChaineJourNuit,"Nuit");
                    }      
                CompteurMesureLuminosite = 0;
                 // Visualisation de la prise d'une mesure de la luminosit sur la led D4
                Ecrire_Port_I2C_Soft(Add_LED_I2C,VisuLectureLum); // Visualisation de la prise de mesure
                delay_ms(50);
                Ecrire_Port_I2C_Soft(Add_LED_I2C,LedsEteintes);
            }
//---------------------------------------------------------------------------------------------------------------
// Transfert des mesures vers le PC par la liaison RS232 toutes les x secondes
//---------------------------------------------------------------------------------------------------------------
        if (CompteurTxRS232 >= 10)  // 10s
            {   // Trame des donnes transfres au PC : NMesure ; TempExt ; TempInt ; Humidit ; Luminosit 
                // Les donnes sont transfres au format CSV (sparateur point-virgule)
                ftoa(TemperatureInterieure_n,1,ChaineTemperatureInterieure);
                ftoa(RH_n,1,ChaineHumidite); 
                itoa((int)CommandeChauffage, ChaineCdeChauffage);
                // Affichage des mesures en clair
                sprintf(TrameRS232Mesures,"Mesure %-i %-c TempExt=%-i.%-uC %-c TempInt=%-sC %-c Hum=%-s%-c %-c %-s %-c P=%-s",
                // Formatage pour transfert dans un fichier 
                // sprintf(TrameRS232Mesures,"%-i%-c%-i.%-u%-c%-s%-c%-s%-c%-c%-u",
                NumeroMesure,';',
                TemperatureExterieure_n/10,abs(TemperatureExterieure_n%10),';',
                ChaineTemperatureInterieure,';',
                ChaineHumidite,'%',';',
                ChaineJourNuit,';', ChaineCdeChauffage);
                //JourOuNuit_n); 
                // Remplacement des . par des , pour rendre les valeurs compatibles avec Excel
                for(i=0;i<sizeof(TrameRS232Mesures);i++)
                {
                    if (TrameRS232Mesures[i]=='.') 
                              TrameRS232Mesures[i]=',';
                }
                
                puts(TrameRS232Mesures); // Transfert sur la liaison RS232
                putchar(0x0D); // retour  la ligne pour l'affichage sur le terminal
                NumeroMesure =  NumeroMesure + 1 ;
                CompteurTxRS232 = 0;
            }
     }
//---------------------------------------------------------------------------------------------------------------
// Fin Acqurir
// ==============================================================================================================
// Dbut Traiter / Communiquer
//---------------------------------------------------------------------------------------------------------------
// Dbut des menus
//---------------------------------------------------------------------------------------------------------------
// Menu [1] : Rglage de l'HTR
//---------------------------------------------------------------------------------------------------------------
    switch (EtatMenu)
    {
        case ReglerHTR : if ((EtatMenu_1 != EtatMenu) || (AfficheMenuReglageHTR))
                            {
        	                    sprintf(display_buffer_ligne0,TexteMenuReglerHTR);
        	                    sprintf(display_buffer_ligne1,TexteMenuHeureOuDate);
        	                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                AfficheMenuReglageHTR = 0;
                                AfficheCaracterePersoSurLCD(FlecheH,15,0);// FlecheH en C15,L0
                                AfficheCaracterePersoSurLCD(FlecheB,15,1);// FlecheB en C15,L1
        		            }

                          switch (BP)
                            {
                                case SET_I2C :  if (PremierReglageHeureHTR)      // Reglage de l'heure
                                            {
        		  	                            RegleHTR(AdresseHTR, ChoixHeure, DefautHeure, DefautMinute, DefautSeconde);
                                                AfficheMenuReglageHTR = 1; PremierReglageHeureHTR = 0;
                                            }
                                            else
                                            {
        		  	                            RegleHTR(AdresseHTR, ChoixHeure, Heures, Minutes, Secondes_n);
                                                AfficheMenuReglageHTR = 1;
                                            }
                                break;

        		                case ENTR_I2C : if (PremierReglageDateHTR)      // Reglage de la date
                                            {
        		  	                            RegleHTR(AdresseHTR, ChoixDate, DefautJour, DefautMois, DefautAnnee);
                                                AfficheMenuReglageHTR = 1; PremierReglageDateHTR = 0;
                                            }
                                            else
                                            {
                                                rtc_get_date(AdresseHTR,&Jour,&Mois,&Annee);
        		  	                            RegleHTR(AdresseHTR, ChoixDate, Jour, Mois, Annee);
                                                AfficheMenuReglageHTR = 1;
                                            }
                                break;

                                case FlecheB_I2C : EtatMenu = Afficher; AfficheMenuReglageHTR = 0; break;

                                case FlecheH_I2C : EtatMenu = Reguler; AfficheMenuReglageHTR = 0; break;

                                default :  EtatMenu = ReglerHTR;
                            }

                            EtatMenu_1 = ReglerHTR;
         break;
//---------------------------------------------------------------------------------------------------------------
// Menu [2] : Affichages 
//          [21] -> Date et heure
//          [22] -> grandeurs physiques
//---------------------------------------------------------------------------------------------------------------
        case Afficher: if (EtatMenu_1 != EtatMenu)
                            {
                                sprintf(display_buffer_ligne0,TexteMenusAfficher);
        	                    sprintf(display_buffer_ligne1,TexteMenuAcquiter);
        	                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                AfficheCaracterePersoSurLCD(FlecheH,15,0);// FlecheH en C15,L0
                                AfficheCaracterePersoSurLCD(FlecheB,15,1);// FlecheB en C15,L1
                            }

                        switch (BP)
                            {
                                case FlecheB_I2C: EtatMenu = Tester; break;
                                case FlecheH_I2C: EtatMenu = ReglerHTR; break;
                                case ENTR_I2C :  EtatMenu = AfficherHorloge; break;
                                default : EtatMenu = Afficher;
                            }

                        EtatMenu_1 = Afficher;
        break;
//---------------------------------------------------------------------------------------------------------------
// Menu [3] : Tests
//          [31] -> Chauffage
//          [32] -> Ventilation
//          [33] -> Trappes
//---------------------------------------------------------------------------------------------------------------
        case Tester: if (EtatMenu_1 != EtatMenu)
                            {
                                sprintf(display_buffer_ligne0,TexteMenusTester);
        	                    sprintf(display_buffer_ligne1,TexteMenuAcquiter);
        	                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                AfficheCaracterePersoSurLCD(FlecheH,15,0);// FlecheH en C15,L0
                                AfficheCaracterePersoSurLCD(FlecheB,15,1);// FlecheB en C15,L1
                            }

                        switch (BP)
                            {
                                case FlecheB_I2C: EtatMenu = Reguler; break;
                                case FlecheH_I2C: EtatMenu = Afficher; break;
                                case ENTR_I2C :  EtatMenu = TesterChauffage; break;
                                default : EtatMenu = Tester;
                            }

                        EtatMenu_1 = Tester;
        break;
//---------------------------------------------------------------------------------------------------------------
// Menu [4] : Rgulation
//---------------------------------------------------------------------------------------------------------------
        case Reguler: if (EtatMenu_1 != EtatMenu)
                            {
                                sprintf(display_buffer_ligne0,TexteMenuRegulerTOR);
        	                    sprintf(display_buffer_ligne1,TexteMenuAcquiter);
        	                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                AfficheCaracterePersoSurLCD(FlecheH,15,0);// FlecheH en C15,L0
                                AfficheCaracterePersoSurLCD(FlecheB,15,1);// FlecheB en C15,L1
                            }

                        switch (BP)
                            {
                                case FlecheB_I2C: EtatMenu = ReglerHTR; break;
                                case FlecheH_I2C: EtatMenu = Tester; break;
                                case ENTR_I2C :  EtatMenu = RegulationTOR; break;
                                default : EtatMenu = Reguler;
                            }

                        EtatMenu_1 = Reguler;
        break;
//---------------------------------------------------------------------------------------------------------------
// Fin des menus
//---------------------------------------------------------------------------------------------------------------
// Dbut des sous-menus
//---------------------------------------------------------------------------------------------------------------
// Sous menu [21] : Affichage de la date et de l'heure
//---------------------------------------------------------------------------------------------------------------
         case AfficherHorloge: if ((EtatMenu_1 != EtatMenu) || (Secondes_n_1H != Secondes_n))
        		                {
        		                    rtc_get_date(AdresseHTR,&Jour,&Mois,&Annee);
        		                    sprintf(display_buffer_ligne0,"[21] %-u/%-u/%-i", Jour, Mois, Annee);
        		                    sprintf(display_buffer_ligne1,"  %-uh/%-umn/%-us", Heures, Minutes, Secondes_n);
        		                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
        	                        Secondes_n_1H = Secondes_n;
                                    AfficheCaracterePersoSurLCD(FlecheH,15,0);// FlecheH en C15,L0
                                    AfficheCaracterePersoSurLCD(FlecheB,15,1);// FlecheB en C15,L1
        	                    }

                                switch (BP)
                                {
                                    case FlecheB_I2C: EtatMenu = AfficherMesures; break;
                                    case FlecheH_I2C: EtatMenu = AfficherMesures; break;
                                    case ECHAP_I2C :  EtatMenu = Afficher; break;
                                    default : EtatMenu = AfficherHorloge;
                                }

        		                EtatMenu_1 = AfficherHorloge;
        break;
//---------------------------------------------------------------------------------------------------------------
// Sous menu [22] : Affichage des mesures sur le LCD
//---------------------------------------------------------------------------------------------------------------
         case AfficherMesures : if ((EtatMenu_1 != EtatMenu) || (TemperatureExterieure_n_1 != TemperatureExterieure_n)
                                                     || (RH_n_1 != RH_n) || (JourOuNuit_n != JourOuNuit_n_1 )
                                                     || (TemperatureInterieure_n_1 != TemperatureInterieure_n))
                                {
                                    // Mise en forme de la valeur de l'humidit pour affichage
        	                        ftoa(RH_n,0,ChaineHumidite);
                                    sprintf(display_buffer_ligne0,"[22]   RH=%-s%-c",ChaineHumidite,0x25);

        	                        // Mise en forme de la valeur des tempratures extrieure et intrieure pour affichage
                                    ftoa(TemperatureInterieure_n,1,ChaineTemperatureInterieure);
                                    sprintf(display_buffer_ligne1,"E=%-i.%-u%-c I=%-s%-c",TemperatureExterieure_n/10,
                                    abs(TemperatureExterieure_n%10),0xdf,ChaineTemperatureInterieure,0xdf);

                                    // Affichage de l'humidit et des tempratures sur le LCD
                                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);

                                    // Affichage de l'icone JourOuNuit sur le LCD
                                    if (JourOuNuit_n == 0) AfficheCaracterePersoSurLCD(Soleil,5,0); // Icne Soleil en C4,L0
                                            else AfficheCaracterePersoSurLCD(Lune,5,0);             // Icne Lune en C4,L0

                                    // Affichage des icnes flche sur le LCD
                                    AfficheCaracterePersoSurLCD(FlecheH,15,0);// Icne FlecheH en C15,L0
                                    AfficheCaracterePersoSurLCD(FlecheB,15,1);// Icne FlecheB en C15,L1

                                    // Mise  jour des variables
                                    RH_n_1 = RH_n; TemperatureExterieure_n_1 = TemperatureExterieure_n;
                                    JourOuNuit_n_1 = JourOuNuit_n; TemperatureInterieure_n_1 = TemperatureInterieure_n;
        		                }

                                switch (BP)
                                {
                                    case FlecheB_I2C: EtatMenu = AfficherHorloge; break;
                                    case FlecheH_I2C: EtatMenu = AfficherHorloge; break;
                                    case ECHAP_I2C :  EtatMenu = Afficher; break;
                                    default : EtatMenu = AfficherMesures;
                                }

        		                EtatMenu_1 = AfficherMesures  ;
         break;
//---------------------------------------------------------------------------------------------------------------
// Sous menu [31] : Test du chauffage (PWM)
//---------------------------------------------------------------------------------------------------------------
        case TesterChauffage :	if (EtatMenu_1 != EtatMenu)
                                 {
                                    sprintf(display_buffer_ligne0,TexteSousMenuChauffage);
                                    sprintf(display_buffer_ligne1,TexteMenuAcquitQuitter);
                                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                    AfficheCaracterePersoSurLCD(FlecheH,15,0);// Icne FlecheH en C15,L0
                                    AfficheCaracterePersoSurLCD(FlecheB,15,1);// Icne FlecheB en C15,L1
                                 }

                                switch(BP)
        		                 {
        		                    case FlecheB_I2C : EtatMenu = TesterVentilation; break;
        		                    case FlecheH_I2C : EtatMenu = TesterTrappes; break;
        		                    case ENTR_I2C:  EtatMenu = CommanderChauffageManu; break;
                                    case ECHAP_I2C: EtatMenu = Tester; break;
        		                    default  : EtatMenu = TesterChauffage;
        		                 }
       			                EtatMenu_1 = TesterChauffage;
        break;
//---------------------------------------------------------------------------------------------------------------
// Sous menu [32] : Test de la ventilation (PWM)
//---------------------------------------------------------------------------------------------------------------
        case TesterVentilation :	if (EtatMenu_1 != EtatMenu)
                                {
                                    sprintf(display_buffer_ligne0,TexteSousMenuVentilation);
                                    sprintf(display_buffer_ligne1,TexteMenuAcquitQuitter);
                                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                    AfficheCaracterePersoSurLCD(FlecheH,15,0);// Icne FlecheH en C15,L0
                                    AfficheCaracterePersoSurLCD(FlecheB,15,1);// Icne FlecheB en C15,L1
                                }

                            switch(BP)
        		                {
        		                    case FlecheB_I2C : EtatMenu = TesterTrappes; break;
        		                    case FlecheH_I2C : EtatMenu = TesterChauffage; break;
        		                    case ENTR_I2C:  EtatMenu = CommanderVentilationManu; break;
                                    case ECHAP_I2C: EtatMenu = Tester; break;
        		                    default  : EtatMenu = TesterVentilation;
        		                }
       			            EtatMenu_1 = TesterVentilation;
        break;
//---------------------------------------------------------------------------------------------------------------
// Sous menu [33] : Test de la commande des trappes (PWM)
//---------------------------------------------------------------------------------------------------------------
        case TesterTrappes :if (EtatMenu_1 != EtatMenu)
                                {
                                    sprintf(display_buffer_ligne0,TexteSousMenuTrappes);
                                    sprintf(display_buffer_ligne1,TexteMenuAcquitQuitter);
                                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                                    AfficheCaracterePersoSurLCD(FlecheH,15,0);// Icne FlecheH en C15,L0
                                    AfficheCaracterePersoSurLCD(FlecheB,15,1);// Icne FlecheB en C15,L1
                                }

                            switch(BP)
        		                {
        		                    case FlecheB_I2C : EtatMenu = TesterChauffage; break;
        		                    case FlecheH_I2C : EtatMenu = TesterVentilation; break;
        		                    case ENTR_I2C:  EtatMenu = CommanderTrappesManu; break;
                                    case ECHAP_I2C: EtatMenu = Tester; break;
        		                    default  : EtatMenu = TesterTrappes;
        		                }
       			            EtatMenu_1 = TesterTrappes;
        break;
 //---------------------------------------------------------------------------------------------------------------
// Sous menu 41: Rgulation TOR
//---------------------------------------------------------------------------------------------------------------
        case RegulationTOR :  if ((EtatMenu_1 != EtatMenu) || (TemperatureInterieure_n_1 != TemperatureInterieure_n))
       	                            {
                                        // Prparation de l'affichage des seuils de temprature et de la temprature
                                        // intrieure sur le LCD
                                        ftoa(TemperatureInterieure_n,1, ChaineTemperatureInterieure);
                                        sprintf(display_buffer_ligne0,"Sn=%-s%-c T=%-s%-c",ChaineSeuilTempIntSimuleMin,0xdf,
       		   	                        ChaineTemperatureInterieure,0xdf);
       		                            sprintf(display_buffer_ligne1,"Sx=%-s%-c P=%-u%-c",ChaineSeuilTempIntSimuleMax,0xdf,
       		   	                        CommandeChauffage,'%');
                                        // Affichage
       		                            Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
       		                            TemperatureInterieure_n_1 = TemperatureInterieure_n;
			                        }

                                    // Commande du chauffage (hystrsis)
                                    if (TemperatureInterieure_n >= SeuilTempIntSimuleMax)
                                     {
                                        PWMChauffage=alphaChauffage[0]; // Cde chauffage = 0%
                                        CommandeChauffage = 0;
                                     }
                                    else if  (TemperatureInterieure_n <= SeuilTempIntSimuleMin)
                                      {
                                        PWMChauffage=alphaChauffage[10];  // Cde chauffage = 100%
                                        CommandeChauffage = 100;
                                      }

                                    if (BP==ECHAP_I2C) {EtatMenu = Reguler; CommandeChauffage = 0; }

         		                    EtatMenu_1 = RegulationTOR;
        break;
//---------------------------------------------------------------------------------------------------------------
//Fin des sous-menus
//---------------------------------------------------------------------------------------------------------------
// Commande manuelle du chauffage
//---------------------------------------------------------------------------------------------------------------
        case CommanderChauffageManu :  if ((EtatMenu_1 != EtatMenu)|| (rcycl_1 != rcycl))
       			                        {
       		   	                            sprintf(display_buffer_ligne0,"Rap. cycl = %-u%%",rcycl);
       		                                sprintf(display_buffer_ligne1,TexteMenuPlusMoinsEsc);
       		                                Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
       		                                rcycl_1 = rcycl;
			                            }
		                               switch(BP)
			                            {
				                            case PLUS_I2C: if (rcycl < 90) rcycl = rcycl+10; else rcycl = 100; break;
				                            case MOINS_I2C: if (rcycl > 10) rcycl = rcycl-10; else rcycl = 0; break;
				                            case ECHAP_I2C : rcycl = 0; EtatMenu = TesterChauffage; break;
				                            default: EtatMenu = CommanderChauffageManu;
                                        }

                                        // Visualisation de l'tat de la commande sur la Led D7

			                            OCR1B=alphaChauffage[rcycl/10];
         		                        EtatMenu_1 = CommanderChauffageManu ;
        break;
//---------------------------------------------------------------------------------------------------------------
// Commande manuelle de la ventilation
//---------------------------------------------------------------------------------------------------------------
        case CommanderVentilationManu :  if ((EtatMenu_1 != EtatMenu)|| (TORVentilation_1 != TORVentilation))
       			                            {
       		   	                                sprintf(display_buffer_ligne0,"Cde Ventila. = %-u",TORVentilation);
       		                                    sprintf(display_buffer_ligne1,TexteMenuPlusMoinsEsc);
       		                                    Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
       		                                    TORVentilation_1 = TORVentilation;
			                                }

		                                 switch(BP)
			                                {
				                                case PLUS_I2C: TORVentilation = 1; break;
				                                case MOINS_I2C: TORVentilation = 0; break;
				                                case ECHAP_I2C : TORVentilation = 0; 
                                                                 EtatMenu = TesterVentilation; 
                                                break;
				                                default: EtatMenu = CommanderVentilationManu;
                                            }

         		                         EtatMenu_1 = CommanderVentilationManu;
        break;
//---------------------------------------------------------------------------------------------------------------
// Commande manuelle des trappes
//---------------------------------------------------------------------------------------------------------------
        case CommanderTrappesManu : do
                                    {
                                      if ((EtatMenu_1 != EtatMenu)||(t1Servo_1 != t1Servo))
       			                        {
       		   	                            sprintf(display_buffer_ligne0,"Servo: t1=%-uus",100*(5+t1Servo));
       		                                sprintf(display_buffer_ligne1,TexteMenuPlusMoinsEsc);
       		                                Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
       		                                t1Servo_1 = t1Servo;
			                            }

		                              BP = Lire_BP_I2C_Soft();

                                       switch(BP)
			                            {
				                            case PLUS_I2C:if (t1Servo < 20) t1Servo = t1Servo+1;
                                                            else t1Servo = 20;
                                            break;
				                            case MOINS_I2C: if (t1Servo > 1) t1Servo = t1Servo-1;
                                                        else t1Servo = 0;
                                            break;
				                            case ECHAP_I2C : t1Servo = 0; EtatMenu = TesterTrappes; break;
				                            default: EtatMenu = CommanderTrappesManu;
                                        }

                                        ReglagePWMServo(t1Servo);
                                      }
                                    while(EtatMenu != TesterTrappes);

         		                    EtatMenu_1 = CommanderTrappesManu;
        break;
//---------------------------------------------------------------------------------------------------------------
        default:     EtatMenu = Afficher;
    }
//---------------------------------------------------------------------------------------------------------------
// Dbut Traiter / Communiquer
//---------------------------------------------------------------------------------------------------------------
 } // fin du while(1)
// ==============================================================================================================
} // fin
// ==============================================================================================================

// ==============================================================================================================
// Sous-programmes
//---------------------------------------------------------------------------------------------------------------
void AfficheCaracterePersoSurLCD(unsigned char numero, unsigned char posColonne, unsigned char posLigne)
{
    lcd_gotoxy(posColonne,posLigne);	// Affichage Jour
    lcd_putchar(numero);
}
//---------------------------------------------------------------------------------------------------------------
unsigned char ReglagePWMServo(unsigned int Reglaget1Servo)
    {
        unsigned char i;
        PWMTrappes = 1;
        for (i=0;i<5+Reglaget1Servo;i++)
        {
            delay_us(100);
        }
        PWMTrappes = 0;
        delay_ms(10);
        return 0;
    }
// ==============================================================================================================